/*--------------------------------------------------------------------*
 | MOSAICD SAS                                                 3.4    |
 |  IML modules for general n-way mosaic display of contingency table.|
 |  This version uses externally calculated cell residuals (dev)      |
 *--------------------------------------------------------------------*
 | Usage:                                                             |
 |  proc iml;                                                         |
 |   %include mosaicd;                                                |
 |   shade = {   }; space = {   }; verbose = ;  see global inputs     |
 |   run mosaicd(levels, table, vnames, lnames, dev, title);          |
 | where:                                                             |
 |   levels    Vector of number of levels of each factor              |
 |   table     Table of cell frequencies, as in IPF: first factor     |
 |             varies most rapidly along cols, last factor varies     |
 |             most slowly down the rows.  If elements of table are   |
 |             a single variable in a SAS data set (e.g., output from |
 |             PROC FREQ), with factors={A B C}, the rows (obs.)      |
 |             should be sorted BY C B A to obtain IPF ordering.      |
 |                                                                    |
 |   vnames    Vector of factor names, order corresponding to levels  |
 |   lnames    Matrix of level names: rows=factors, cols=max(levels)  |
 |             lnames[i,1:levels[i]] gives level names for factor i.  |
 |                                                                    |
 |   dev       Deviations from model                                  |
 |   title     Character string title for plots                       |
 |                                                                    |
 | Global input variables (optional)                                  |
 |   filltype  Type of fill pattern to use for shading.               |
 |               'LR'  --> patterns Ld, Rd, where d=density value     |
 |               'M0'  --> patterns MdN0, MdN90                       |
 |               'M45' --> patterns MdN135, Md45 [default]            |
 |   htext     Height of text labels [default: 1.3]                   |
 |   shade     Vector of up to 5 values of abs(dev[i]) for boundaries |
 |             between shading levels. If shade={2 4}; (the default), |
 |             then shading density is:                               |
 |                   0 <= |dev[i]| < 2  ->  0 (empty)                 |
 |                   2 <= |dev[i]| < 4  ->  1                         |
 |                   4 <= |dev[i]|      ->  2                         |
 |             Use shade= a big number to suppress all shading.       |
 |   space     2-vector of x,y: amount of plotting area reserved for  |
 |             spacing. Typically {20 20}.                            |
 |   verbose   Controls verbose or detailed output                    |
 *--------------------------------------------------------------------*
 *  Author:  Michael Friendly            <friendly@yorku.ca>          *
 * Created:  11 Aug 1990 10:27:11          (c) 1990, 1991             *
 * Revised:  13 May 1998 09:00:11                                     *
 * Version:  3.4                                                      *
 *--------------------------------------------------------------------*/
*title 'Mosaic displays for externally-fitted models';
*version(6.06);  *-- Requires SAS Version 6.06 or later;
 
*proc iml;
start mosaicd(levels, table, vnames, lnames, dev, title)
      global(config, devtype, fittype, filltype, shade, space, split,
             legend, colors, htext, font, verbose, cellfill, order, zeros);
 
   if nrow(vnames)=1 then vnames=vnames`;
   if nrow(levels)=1 then levels=levels`;
   print / '+-------------------------------------------+',
           '|  Specialized Mosaic Display, Version 3.4  |',
           '+-------------------------------------------+',,
           title, vnames levels '  ' lnames;
 
   *-- Check conformability of arguments --;
   f = nrow(vnames)||nrow(lnames);
   if ^all(f = nrow(levels))
      | levels[#] ^= nrow(table)#ncol(table)
      then do;
         print 'Arguments not conformable'; show levels table;
         goto done;
      end;
 
	run globals;	
	reset name;
   print 'Global options', filltype split
                           shade[f=3.0] colors ,htext
									font legend cellfill verbose;
   f = nrow(levels);
   whichway = num(translate(split,'10','HV'));
   dir = shape(whichway,f,1);
   if type(space) ^= 'N'
      then space = 10# (sum(dir=0) || sum(dir=1));
   savspace = space;
	savedev = dev;
	 
   call gstart;
   *-- divide the plot into boxes  --;
   run divided(levels, table, vnames, lnames, dev, dir, title);
   call gstop;
   space = savspace;
	dev = savedev;
   done:
   finish;
 
start divided(levels, table, vnames, names, dev, dir, title)
      global(shade, space, verbose);
   *-- start with origin in lower left corner --;
   length= {100 100};                /* x,y length of box area */
   boxes = {0 0}                     /* lowerleft  x,y  */
         ||( length - space );       /* length     x,y  */
   factors = nrow( levels );
   rows = 1;                 /* number of rows in margins */
   do f = 1 to factors ;
      whichway =dir[f];
      cols = levels[f];      /* number of cols in margins */
      reset noname;
      print "Factor:" f (vnames[f]);
      reset name;
      mconfig = (f:1)`;       * n sub f, (f-1), ..., 1 ;
      call marg(loc,margin,levels,table,mconfig);
 
      *-- Construct row and column labels for marginal table;
      cl = names[f,];
      if f=1 then rl='';
         else do;        /* construct row labels for prior factors */
            ol = repeat(rl, 1, levels[f-1]);
            ol = shape(ol,  levels[f-1]#nrow(rl), 1);
            nl = repeat((names[f-1,1:(levels[f-1])])`, nrow(rl));
            rl = concat(ol,nl);
         end;
      margin = (shape(margin, rows)) ;
      print 'Marginal totals', margin[r=rl c=cl ];
	  
 
      *-- Calculate proportions for each row over row totals, allowing
          for 0 margins;
      mar = row( margin );
      margin = margin / ( ( margin[,+] + (0=margin[,+]) )
                            * J(1,levels[f]) );
      *-- Divide boxes for this factor;
      run divide1(levels, margin, boxes, f, whichway);
 
      if any(verbose = 'BOX') then do;
         bl = shape( (rl || J(rows,cols-1,' ')),rows#cols,1);
         print boxes[r=bl c={'BotX' 'BotY' 'LenX' 'LenY'} format=8.2];
      end;
 
      run  space( levels, boxes, f, dir );
      run labels( levels, vnames, names, f, dir, boxes, labelx, labels );
 
      if any(verbose = 'BOX') then;
      print labelx[r=labels c={x y 'Angle' 'Height'}] ;
 
      *-- display the mosaic for current margins ;
      if f=factors then do;
		  res=shape(dev,rows);
	      print 'Residuals', res[r=rl c=cl f=8.2];
         *-- at this point the marginal table has been turned inside-out;
         *   so, dev must be reordered to match. However, MARG chokes on
             values < 0, so subtract minimum value, then add it back;
         mdev = min(dev);
         run marg(loc,newdev,levels, dev-mdev, mconfig);
         dev = newdev + mdev;
*?????   dev = shape( dev,1,(nrow(dev)#ncol(dev)) );
         call gboxes( boxes, labels, labelx, dev, f, title, mar );
         run gskip;
         end;
 
      *-- prepare for next factor;
      rows = rows * levels[f];
   end;
   finish;
 
*reset storage=mosaic.mosaic;
*store module=_all_;
*show storage;
